// WiFiDataHandling.cpp
/*
 * (c)2024 Forward Computing and Control Pty. Ltd.
 * NSW Australia, www.forward.com.au
 * This code is not warranted to be fit for any purpose. You may only use it at your own risk.
 * This generated code may be freely used for both private and commercial use
 * provided this copyright is maintained.
 */
/**
   This file handles incomming commands and outgoing data
   Settings/data transferred to the loop() via volatile vars listed in volatileVars.cpp / h
*/
#include <Arduino.h>
#include "WiFiDataHandling.h"
#include "VolatileVars.h"
#include "millisDelay.h"
#include "SafeString.h"

static unsigned long lastLoopCount;
static unsigned long last_us;

static millisDelay dataTimer;
static unsigned long DATA_TIMER_MS = 1000; // every 2sec

/**
   asyncSetup
   runs on WiFi core 0, in HS_Async thread (task). Use volatiles to interact with your Arduino loop() code
   Called just once when the HS_AsyncTCP task on core 0 starts
   Use this to do any initialization of WiFi task local variables, e.g. start timers etc
*/
void asyncSetup() {
  dataTimer.start(DATA_TIMER_MS);
}

// this is called from core 1 asyncTCP task
// approximately every 1.5ms, but can be delayed upto ~20ms if the core 1 is busy with the WiFi
static uint32_t lastRainPulses = 0;
void asyncLoop(Stream &stream) {
  if (dataTimer.justFinished()) {
    dataTimer.restart();
    bool isRaining = isRaining_v;
    uint32_t isRainingCounter = isRaining_counter_v;
    uint32_t rainPulses = rainPulses_v;
    uint32_t deltaT_counter = deltaT_counter_v;
    float maxRatio = maxRatio_v;
    maxRatio_v = 0;
    float maxRatioMagnitude = maxRatioMagnitude_v;
    maxRatioMagnitude_v = 0;
    stream.print(isRainingCounter);
    stream.print(",");
    if (isRaining) {
      stream.print("Rain, ");
    } else {
      stream.print("Dry, ");
    }
    stream.print(deltaT_counter);
    stream.print(" , ");
    stream.print(rainPulses-lastRainPulses);
    stream.print(" , ");
    stream.print(maxRatio,1);
    stream.print(" , ");
    stream.print(maxRatioMagnitude,0);
    stream.println();
    lastRainPulses = rainPulses;
  }
}
// msg to send back when connection opened
void asyncConnected(Stream &stream) {
  stream.println("Results output every 1sec.");
  stream.println("last change count, State, current count(*0.05s), pulses/sec, maxRatio, maxRatioMag");
}

void asyncDisconnected() {
  // nothing here at the moment
}

// this is called from core 1 by the WiFi support
void asyncDataReceived(Stream &stream) {
  // no cmds for now
}
